@@ -3,9 +3,9 @@
 
 # "Repairing Code Automatically" - a chapter of "The Debugging Book"
 # Web site: https://www.debuggingbook.org/html/Repairer.html
-# Last change: 2021-10-13 13:42:28+02:00
+# Last change: 2023-01-04 15:49:38+01:00
 #
-# Copyright (c) 2021 CISPA Helmholtz Center for Information Security
+# Copyright (c) 2021-2023 CISPA Helmholtz Center for Information Security
 # Copyright (c) 2018-2020 Saarland University, authors, and contributors
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
@@ -65,13 +65,7 @@ print(ast.unparse(tree), fitness)
 Here is a complete example for the `middle()` program. This is the original source code of `middle()`:
 
 def middle(x, y, z):  # type: ignore
-    if y < z:
-        if x < y:
-            return y
-        elif x < z:
-            return y
-    else:
-        if x > y:
+    if y  y:
             return y
         elif x > z:
             return x
@@ -96,12 +90,7 @@ The returned AST `tree` can be output via `ast.unparse()`:
 
 >>> print(ast.unparse(tree))
 def middle(x, y, z):
-    if y < z:
-        if x < y:
-            return y
-        elif x < z:
-            return x
-    elif x > y:
+    if y  y:
         return y
     elif x > z:
         return x
@@ -461,7 +450,7 @@ class StatementMutator(StatementMutator):
 
         return suspiciousness
 
-    def format_node(self, node: ast.AST) -> str:
+    def format_node(self, node: ast.AST) -> str:  # type: ignore
         ...
 
 class StatementMutator(StatementMutator):
@@ -960,7 +949,7 @@ class CrossoverOperator(CrossoverOperator):
             return False
 
         body = getattr(tree, body_attr, [])
-        return body and len(body) >= 2
+        return body is not None and len(body) >= 2
 
 class CrossoverOperator(CrossoverOperator):
     def crossover_attr(self, t1: ast.AST, t2: ast.AST, body_attr: str) -> bool:
