@@ -8,7 +8,7 @@ class PrettyTable(list):
         IPython Notebook. For LaTeX export two styles can be chosen."""
     
     def __init__(self, initlist: Any = [],
-                 extra_header: List[str] = None,
+                 extra_header: Optional[List[str]] = None,
                  print_latex_longtable: bool = True) -> None:
         self.print_latex_longtable = print_latex_longtable
         if extra_header is not None:
