@@ -9,6 +9,7 @@ from IPython import get_ipython
 from nbformat import read
 from IPython.core.interactiveshell import InteractiveShell
 from importlib.abc import MetaPathFinder
+from importlib.machinery import ModuleSpec
 
 import linecache
 import ast
@@ -40,6 +41,8 @@ assert do_import("def foo():\n    pass")
 assert do_import("# ignore\ndef foo():\n    pass")
 assert do_import("# ignore\nclass Bar:\n    pass")
 assert do_import("XYZ = 123")
+assert do_import("Timeout = A if f() else B")
+assert do_import("Zoo: Set[Animal] = {...}")
 assert not do_import("xyz = 123")
 assert not do_import("foo()")
 
@@ -127,10 +130,11 @@ class NotebookFinder(MetaPathFinder):
     def __init__(self) -> None:
         self.loaders: Dict[Any, Any] = {}
 
+    # Deprecated since Python 3.4
     def find_module(self, fullname: str, path: Any = None) -> Any:
         nb_path = find_notebook(fullname, path)
         if not nb_path:
-            return
+            return None
 
         key = path
         if path:
@@ -140,5 +144,13 @@ class NotebookFinder(MetaPathFinder):
         if key not in self.loaders:
             self.loaders[key] = NotebookLoader(path)
         return self.loaders[key]
+        
+    # New since Python 3.4
+    def find_spec(self, fullname: str, path: Any = None, target: Any = None) -> Any:
+        loader = self.find_module(fullname, path)
+        if not loader:
+            return None
+        
+        return ModuleSpec(fullname, loader)
 
 sys.meta_path.append(NotebookFinder())
